package com.lordjoe.dynamiccompile;

import javax.tools.*;
import java.util.*;

/**
 * com.lordjoe.dynamiccompile.ClassBuilder
 *
 * @author Steve Lewis
 * @date Feb 21, 2010
 * @see http://www.javablogging.com/dynamic-in-memory-compilation/
 */
public class ClassBuilder
{
    public static ClassBuilder[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = ClassBuilder.class;

    private static ClassBuilder gInstance;

    public static synchronized ClassBuilder getInstance()
    {
        if (gInstance == null)
            gInstance = new ClassBuilder();
        return gInstance;
    }

    private final Map<String, Class> m_Classes;
    private final JavaCompiler m_Compiler;
    private final JavaFileManager m_FileNamager;

    private ClassBuilder()
    {
        m_Classes = new HashMap<String, Class>();
        m_Compiler = ToolProvider.getSystemJavaCompiler();
        m_FileNamager =  new
                ClassFileManager(m_Compiler
                .getStandardFileManager(null, null, null));
    }

    public JavaCompiler getCompiler()
    {
        return m_Compiler;
    }

    public JavaFileManager getFileNamager()
    {
        return m_FileNamager;
    }

    public Object getInstance(String className)  {
        try {
            Class<?> aClass = getClassByName(className);

            Object instance = aClass.newInstance();
            return instance;
        }
        catch (InstantiationException e) {
            throw new RuntimeException(e);
        }
        catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    public Class<?> getClassByName(String className)
     {
         try {
             Class ret = m_Classes.get(className);
             if(ret != null)
                     return ret;
// Creating an instance of our compiled class and
             // running its toString() method
             ClassLoader loader = getFileNamager().getClassLoader(null);
             Class<?> aClass = loader.loadClass(className);
             m_Classes.put(className,aClass);
             return aClass;
         }
         catch (ClassNotFoundException e) {
             throw new RuntimeException(e);
         }
     }

    public Boolean buildClassFromText(String className,String text)
    {
        // Dynamic compiling requires specifying
          // a list of "files" to compile. In our case
          // this is a list containing one "file" which is in our case
          // our own implementation (see details below)
          List<JavaFileObject> jfiles = new ArrayList<JavaFileObject>();
          jfiles.add(new CharSequenceJavaFileObject(className, text));

          // We specify a task to the compiler. Compiler should use our file
          // manager and our list of "files".
          // Then we run the compilation with call()
        JavaCompiler compiler = getCompiler();

        /**
         * add the current classpath
         */
        List<String> optionList = new ArrayList<String>();
        optionList.addAll(Arrays.asList("-classpath",System.getProperty("java.class.path")));
       // optionList.addAll(Arrays.asList(options));
        JavaCompiler.CompilationTask compilationTask = compiler.getTask(null, getFileNamager(),
                null, optionList,
                null, jfiles);
        Boolean aBoolean = compilationTask.call();
        return aBoolean;
    }


    public String findClassNameFromText(String text) {
        String packageStr = null;
         if(text.indexOf("package") > -1)
             packageStr =  extractBetween("package",";",text);
        String classStr = extractBetween(" class "," ",text).trim();
        if(packageStr == null)
            return classStr;
        else
            return packageStr + "." + classStr;
    }


    public Boolean buildClassFromText( String text)
    {
        String nameFromText = findClassNameFromText(text);
        return buildClassFromText(nameFromText,text);

    }


        /**
     * find the text between start and end strings
     *
     * @param start
     * @param end
     * @param text  string to search
     * @return
     * @throws IllegalArgumentException if not found
     */
    public static String extractBetween(String start, String end, String text)
            throws IllegalArgumentException
    {
        int startIndex = text.indexOf(start);
        if (startIndex == -1)
            throw new IllegalArgumentException("string " + start + " not found");
        startIndex += start.length();
        int endIndex = text.indexOf(end, startIndex);
        if (endIndex == -1)
            throw new IllegalArgumentException("string " + end
                    + " not found after " + start);
        return text.substring(startIndex, endIndex);
    }

    public static void main(String[] args) throws Exception
    {
        // Full name of the class that will be compiled.
        // If class should be in some package,
        // fullName should contain it too
        // (ex. "testpackage.DynaClass")
        String fullName = "DynaClass";

        // Here we specify the source code of the class to be compiled
        StringBuilder src = new StringBuilder();
        src.append("public class DynaClass {\n");
        src.append("    public String toString() {\n");
        src.append("        return \"Hello, I am \" + ");
        src.append("this.getClass().getSimpleName();\n");
        src.append("    }\n");
        src.append("}\n");

        ClassBuilder cb =  ClassBuilder.getInstance();

        String text = src.toString();
        String className = cb.findClassNameFromText(text);
        Boolean success = cb.buildClassFromText(text);

        Object instance = cb.getInstance(className);
        System.out.println(instance);
    }
}




