package com.lordjoe.formgenerator;

import com.google.api.translate.*;

/**
 * com.lordjoe.formgenerator.FormGenerator
 *
 * @author Steve Lewis
 * @date Jan 18, 2010
 */
public class FormGenerator
{
    public static FormGenerator[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = FormGenerator.class;

    public static final String STANDARD_ODK_HEADER =
            "<h:html xmlns=\"http://www.w3.org/2002/xforms\"\n" +
                    "        xmlns:h=\"http://www.w3.org/1999/xhtml\"\n" +
                    "        xmlns:ev=\"http://www.w3.org/2001/xml-events\"\n" +
                    "        xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\"\n" +
                    "        xmlns:jr=\"http://openrosa.org/javarosa\">";
    private final String m_Title;
    private final FormModel m_DTO;
    private final Language[] m_Translations;

    public FormGenerator(FormModel pDTO, Language[] translations)
    {
        m_Title = pDTO.getName();
        m_DTO = pDTO;
        m_Translations = translations;
    }

    public FormGenerator(Class pDTO, Language[] translations)
    {
        this(Utilities.fromClass(pDTO),translations);
    }

    public String generateForm()
    {
        StringBuilder sb = new StringBuilder();
        int indent = 0;
        appendHeader(sb, indent);
        appendBody(sb, indent + 1);
        appendFooter(sb, indent);
        return sb.toString();
    }

    public String getTitle()
    {
        return m_Title;
    }

    public FormModel getDTO()
    {
        return m_DTO;
    }

    public static void indent(StringBuilder sb, int indent)
    {
        sb.append("\n");
        for (int i = 0; i < indent; i++) {
            sb.append("    ");

        }
    }


    protected void appendHeader(StringBuilder sb, int indent)
    {
        sb.append(STANDARD_ODK_HEADER);
        indent(sb, indent++);
        sb.append("<h:head>");
        indent(sb, indent);
        sb.append("<h:title>");
        sb.append(getTitle());
        sb.append("</h:title>");
        indent(sb, indent);
        appendText(sb, indent + 2);
        appendModel(sb, indent + 2);
        indent(sb, indent);
        sb.append("</h:head>");
        indent(sb, indent);

    }

    protected void appendText(StringBuilder sb, int indent)
    {
        sb.append("<itext>");
        indent(sb, indent);
        appendEnglish(sb, indent + 1);

        sb.append("</itext>");
        indent(sb, indent);

    }

    protected void appendEnglish(StringBuilder sb, int indent)
    {
        sb.append("<translation lang=\"English\">");
        indent(sb, indent);
        appendEnglishItems(sb, indent + 1);

        sb.append("</translation>");
        indent(sb, indent);

    }

    protected void appendEnglishItems(StringBuilder sb, int indent)
    {
        sb.append("<text id=\"English\">");
        indent(sb, indent);
         // todo append value

        sb.append("</text>");
        indent(sb, indent);

    }

    protected void appendModel(StringBuilder sb, int indent)
    {
        sb.append("<model>");
        indent(sb, indent);
        appendInstance(sb, indent + 1);
        appendBindings(sb, indent + 1);

        sb.append("</model>");
        indent(sb, indent);

    }

    protected void appendInstance(StringBuilder sb, int indent)
    {
        sb.append("<instance>");
        indent(sb, indent);
        appendInstance(sb, indent + 1);

        sb.append("</instance>");
        indent(sb, indent);

    }

    protected void appendBody(StringBuilder sb, int indent)
    {
        sb.append("<h:body>");
        indent(sb, indent++);
        sb.append("</h:body>");
        indent(sb, indent);

    }

    protected void appendBindings(StringBuilder sb, int indent)
    {

    }

    protected void appendFooter(StringBuilder sb, int indent)
    {

    }
}
