package com.lordjoe.formgenerator;

import com.google.api.translate.*;
import com.lordjoe.translation.*;

import java.io.*;
import java.util.*;

/**
 * com.lordjoe.formgenerator.FormModel
 *
 * @author Steve Lewis
 * @date Jan 18, 2010
 */
public class FormModel extends AbstractFormGroup
{
    public static FormModel[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = FormModel.class;

    public static final String FORM_HEADER =
            "<?xml version=\"1.0\"?>\n" +
                    "<h:html xmlns=\"http://www.w3.org/2002/xforms\" \n" +
                    "\t\t\t\txmlns:h=\"http://www.w3.org/1999/xhtml\" \n" +
                    "\t\t\t\txmlns:ev=\"http://www.w3.org/2001/xml-events\" \n" +
                    "\t\t\t\txmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" \n" +
                    "\t\t\t\txmlns:jr=\"http://openrosa.org/javarosa\">";

    private final List<TranslationItem> m_Items = new ArrayList<TranslationItem>();

    public FormModel(String pName)
    {
        super(null);
        setName(pName);
    }

    protected String getPathName()
    {
      //  return "";
       return "/" + getName().replace(" ","_");
     }


    public TranslationItem[] getTranslationItems()
    {
        return m_Items.toArray(TranslationItem.EMPTY_ARRAY);
    }

    @Override
    protected void accumulateTranslations(Set<String> ids)
    {
        super.accumulateTranslations(ids);
        ids.add(getName());
    }


    public String generateXForm(Language[] langs)
    {
        StringBuilder sb = new StringBuilder();
        sb.append(FORM_HEADER);
        sb.append("\n");
        appendHeader(langs, sb, 1);
        appendBody(sb, 1);
        sb.append("</h:html>\n");
        return sb.toString();
    }

    protected void appendHeader(Language[] langs, StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<h:head>\n");

        FormUtil.indent(sb, indent++);
        sb.append(" <h:title>" + getName() + "</h:title>\n");

        appendModel(langs,sb, indent);

        FormUtil.indent(sb, --indent);
        sb.append("</h:head>\n");
    }

    protected void appendModel(Language[] langs,StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<model>\n");

        appendTranslations(langs, sb, indent);

        appendInstance(sb, indent);

        FormUtil.indent(sb, --indent);
        sb.append("</model>\n");
    }


    @Override
    protected void appendInstance(StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<instance>\n");

        FormUtil.indent(sb, indent++);
        String s = getName().replace(" ","_");
        sb.append("<" + s + ">\n");

        super.appendInstance(sb, indent);


        FormUtil.indent(sb, --indent);
        sb.append("</" + s + ">\n");

        FormUtil.indent(sb, --indent);
        sb.append("</instance>\n");
        appendBinding(sb, indent);
    }


    protected void appendTranslations(Language[] langs, StringBuilder sb, int indent)
    {
        buildTranslations();
        TranslationItem[] items = getTranslationItems();
        File udir = new File(System.getProperty("user.dir"));
        File formsdDir = new File(udir,"form_resources");
        File formdDir = new File(formsdDir,getName());
        formdDir.mkdirs();
         ResourceTranslator rt = buildTranslationResources(formdDir,items, langs);

        Map<Language, LanguageBundle> map = rt.getLanguageBundles(langs,true);
        FormUtil.indent(sb, indent++);
        sb.append("<itext>\n");
        for (Language lang : langs) {
            Translation translation = new Translation(map.get(lang));
            appendTranslation(translation,  sb,  indent);

        }
         FormUtil.indent(sb, --indent);
        sb.append("</itext>\n");
    }

    protected void buildTranslations()
    {
        Set<String> englishValues = new HashSet<String>();
        accumulateTranslations(englishValues);
        String[] vals = englishValues.toArray(new String[0]);
        Arrays.sort(vals);
        m_Items.clear();
        for (int i = 0; i < vals.length; i++) {
            String s = vals[i];
            String key = FormUtil.textToKey(s);
            String value = FormUtil.forXML(FormUtil.camelCaseToWords(s));
            m_Items.add(new TranslationItem(key, value));
        }

    }

    protected ResourceTranslator buildTranslationResources(File pBaseDirectory,TranslationItem[] items, Language[] langs)
    {
         ResourceTranslator rt = new ResourceTranslator(pBaseDirectory);
        Language baseLanguage = rt.getBaseLanguage();
        final String suffix = "_" + baseLanguage.toString() + ".properties";
        File baseFile = new File(pBaseDirectory,getName() + suffix);
        LanguageBundle base = new LanguageBundle(baseLanguage, baseFile);
        boolean altered = false;
        if (baseFile.exists()) {
            base.load();
        }
        else {
            altered = true;
        }
        for (int i = 0; i < items.length; i++) {
            TranslationItem item = items[i];
            TranslationEntry te = base.getTranslationEntry(item.getID());
            if (te == null) {
                altered = true;
                base.setProperty(item.getID(), item.getValue());
                base.getTranslationEntry(item.getID()).setHandEdited(
                        true); // base language is hand edited
            }
        }
        if (altered)
            base.save();
        rt.translateDirectory(langs);
        return rt;
    }


    protected void appendTranslation(Translation tr, StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<translation lang=\"" + tr.getLanguage() + "\" >\n");
        TranslationItem[] translationItems = tr.getTranslationItems();
        for (TranslationItem itm : translationItems)
            appendTranslationItem(itm, sb, indent);
        FormUtil.indent(sb, --indent);
        sb.append("</translation>\n");
    }

    protected void appendTranslationItem(TranslationItem tr, StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<text id=\"" + tr.getID() + "\" >");
        sb.append("<value>" + FormUtil.forXML(tr.getValue()) + "</value>");
        sb.append("</text>\n");
    }


    protected void appendBody(StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        sb.append("<h:body>\n");

        appendInterface(sb, indent);

        FormUtil.indent(sb, --indent);
        sb.append("</h:body>\n");
    }


}
