package com.lordjoe.formgenerator;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * com.lordjoe.formgenerator.FormObject
 *
 * @author Steve Lewis
 * @date Jan 29, 2010
 */
public abstract class FormObject implements Serializable
{
    public static FormObject[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = FormObject.class;

    public static final String REQUIRED_PROPERTY = "Required";
    public static final String RELEVANT_PROPERTY = "Relevant";
    public static final String OPTIONS_PROPERTY = "Options";
    public static final String CALCULATE_PROPERTY = "Calculate";
    public static final String QUESTION_PROPERTY = "Question";
    public static final String PRONOUN_PROPERTY = "Pronoun";
    public static final String COMPOSITE_PROPERTY = "Composite";
    public static final String RESOURCE_PROPERTY = "Resources";


    private final FormObject m_Parent;
    private String m_Name;
    private String m_RelevantIf;
    private Map<String, Object> m_Properties = new HashMap<String, Object>();

    public QuestionPossessive getDefaultQuestionPronoun()
    {
        QuestionPossessive qp = (QuestionPossessive)getProperty(PRONOUN_PROPERTY)  ;
        if(qp != null)
            return qp;
        FormObject parent = getParent();
        if (parent != null)
            return parent.getDefaultQuestionPronoun();
        return QuestionPossessive.Your;
    }

    public FormObject(FormObject pParent)
    {
        m_Parent = pParent;
    }

    public String getRelevantIf()
    {
        return (String) getProperty(RELEVANT_PROPERTY);

    }


    public String getBindingAllotations()
    {
        StringBuilder sb = new StringBuilder();
        String relevantIf = getRelevantIf();
        if (relevantIf != null) {
            sb.append(" relevant=\"" + relevantIf + "\" ");
        }

        return sb.toString();

    }


    public String buildFormQuestion(Field fld)
    {
        QuestionPossessive possessive = getDefaultQuestionPronoun();
        Class type = fld.getType();
        if (type == Boolean.TYPE || type == Boolean.class) {
            switch (possessive) {
                case Your:
                    return "Are you " + getName();
                case Her:
                    return "Is he " + getName();
                case His:
                    return "Is she " + getName();
                case Its:
                    return "Is it " + getName();

            }
        }
        if (Date.class.isAssignableFrom(type)) {
            return "When is " + possessive + " " + getName();
        }
        if (type.isArray())
            return "What are " + possessive + " " + getName();
        return "What is " + possessive + " " + getName();
    }

    public void addProperty(String name, Object value)
    {
        m_Properties.put(name, value);
    }

    public Object getProperty(String name)
    {
        return m_Properties.get(name);
    }

    public String getName()
    {
        return m_Name;
    }

    protected String getPathName()
    {
        return getName();
    }

    public void setName(String pName)
    {
        m_Name = pName;
    }


    public FormObject getParent()
    {
        return m_Parent;
    }

    public String getDataPath()
    {
        StringBuilder sb = new StringBuilder();
        FormObject parent = getParent();
        String base = "";
        if (parent != null)
            base = parent.getDataPath();
        if (base.length() > 0)
            sb.append(base + "/");
        sb.append(getPathName());
        return sb.toString();
    }

    protected void accumulateTranslations(Set<String> ids)
    {
        ids.add(getName());
    }

    protected String getInterfaceAttributes()
    {
        StringBuilder sb = new StringBuilder();

        return sb.toString();
    }

    protected String getBindingAttributes()
    {
        StringBuilder sb = new StringBuilder();

        return sb.toString();
    }

    protected abstract void appendInstance(StringBuilder sb, int indent);

    protected abstract void appendBinding(StringBuilder sb, int indent);

    protected abstract void appendInterface(StringBuilder sb, int indent);


}
