package com.lordjoe.formgenerator;

import java.util.*;

/**
 * com.lordjoe.formgenerator.FormProperty
 *
 * @author Steve Lewis
 * @date Jan 18, 2010
 */
public class FormProperty extends FormObject
{
    public static FormProperty[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = FormProperty.class;


    private FormType m_Type;
    private String m_LabelId;
    private String m_QuestionId;

    public FormProperty(FormObject pParent)
    {
        super(pParent);
    }

    public boolean isRequired()
    {
        Boolean required = (Boolean) getProperty(REQUIRED_PROPERTY);
        return (required == Boolean.TRUE);

    }


    public void setName(String pName)
    {
        super.setName(pName);
    }


    public FormType getType()
    {
        return m_Type;
    }

    public void setType(FormType pType)
    {
        m_Type = pType;
    }

    public String getLabelId()
    {
        return m_LabelId;
    }

    public void setLabelId(String pLabelId)
    {
        m_LabelId = pLabelId;
    }

    public String getQuestionId()
    {
        return m_QuestionId;
    }

    public void setQuestionId(String pQuestionId)
    {
        m_QuestionId = pQuestionId;
    }

    @Override
    protected void appendInstance(StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent);
        sb.append("<");
        sb.append(getName());
        sb.append("/>\n");
    }


    @Override
    protected void appendBinding(StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent);
        sb.append("<bind nodeset=\"");
        sb.append(getDataPath());
        sb.append("\" type=\"");
        sb.append(getXFormType());
        sb.append("\" ");
        sb.append(getBindingAllotations());

        sb.append(" />\n");
    }


    public String getBindingAllotations()
    {
        StringBuilder sb = new StringBuilder(super.getBindingAttributes());
        String relevantIf = getRelevantIf();
        if (isRequired()) {
            sb.append(" required=\"true()\"");
        }

        return sb.toString();

    }


    public String getXFormType()
    {
        FormType type = getType();
        return type.getXFormType();
    }

    @Override
    protected void appendInterface(StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent++);
        FormType type = getType();
        String controlType = "input";
        AllowedType allowedType = type.getType();
        switch (allowedType) {
            case Number:
                break;
            case Boolean:
            case Select1:
                controlType = "select1";
                break;
            case SelectMulti:
                controlType = "select";
                break;
        }

        sb.append("<" + controlType + "  ref=\"" + getDataPath() + "\" >\n");
        FormUtil.indent(sb, indent);

        String labelId = FormUtil.textToKey(getLabelId());
        sb.append("<label ref=\"jr:itext(\'" + labelId + "\')\" />\n");

        IOptionSet options = (IOptionSet) getProperty(OPTIONS_PROPERTY);
        if (allowedType == AllowedType.Boolean) {
            options = OptionSet.getBooleanSet();
        }

        if (options != null) {
            appendOptions(options, sb, indent);
        }

        FormUtil.indent(sb, --indent);
        sb.append("</" + controlType + ">\n");

    }

    @Override
    protected void accumulateTranslations(Set<String> ids)
    {
        super.accumulateTranslations(ids);
        ids.add(getLabelId());
        IOptionSet options = (IOptionSet) getProperty(OPTIONS_PROPERTY);
        FormType type = getType();
        AllowedType allowedType = type.getType();
        if (allowedType == AllowedType.Boolean) {
            options = OptionSet.getBooleanSet();
        }
        if (options != null) {
            for (IOption opt : options.getOptions()) {
                String s = opt.getId();
                ids.add(s);
            }
        }

    }


    protected void appendOptions(IOptionSet options, StringBuilder sb, int indent)
    {
        for (IOption opt : options.getOptions()) {
            appendOption(opt, sb, indent);
        }
    }

    protected void appendOption(IOption opt, StringBuilder sb, int indent)
    {
        FormUtil.indent(sb, indent);
        sb.append("<item>\n");
        FormUtil.indent(sb, indent + 1);
        sb.append("<value>");
        sb.append(opt.getValue());
        sb.append("</value>\n");
        FormUtil.indent(sb, indent + 1);
        sb.append("<label ref=\"jr:itext(\'");
        sb.append(opt.getId());
        sb.append("')\" />\n");
        FormUtil.indent(sb, indent);
        sb.append("</item>");
        sb.append("\n");
    }

}
