package com.lordjoe.formgenerator;

import java.lang.reflect.*;
import java.text.*;
import java.io.*;
import java.util.*;

/**
 * com.lordjoe.formgenerator.FormUtil
 *
 * @author Steve Lewis
 * @date Jan 29, 2010
 */
public class FormUtil
{
    public static FormUtil[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = FormUtil.class;

    public static void indent(StringBuilder sb, int indent)
    {
        for (int i = 0; i < indent; i++) {
            sb.append("    ");

        }
    }

    public static Class[] getGenericTypes(Field f)
    {
        if (!isFieldTypeGeneric(f))
            return null;
        Type t2 = f.getGenericType();

        if (t2 instanceof ParameterizedType) {
            ParameterizedType type = (ParameterizedType) t2;
            Type[] typeArguments = type.getActualTypeArguments();
            List<Class> holder = new ArrayList<Class>();

            for (Type typeArgument : typeArguments) {
                if (typeArgument instanceof Class) {
                    holder.add((Class) typeArgument);
                }
            }

            Class[] ret = new Class[holder.size()];
            holder.toArray(ret);
            return ret;
        }
        return null;
    }

    /**
     * See  http://www.ibm.com/developerworks/java/library/j-cwt11085.html#h2
     *
     * @param f non-null field
     * @return field type including generics adjusted for C# which is pickier about these things
     */
    public static boolean isFieldTypeGeneric(Field f)
    {
        Class cls = f.getType();
        Type type = f.getGenericType();
        if (type == null)
            return false;
        if (type instanceof ParameterizedType) {
            return true;
        }
        if (type instanceof GenericArrayType) {
            // todo handle this case
            return true;
        }
        if (type instanceof TypeVariable) {
            // todo handle this case
            return true;
        }
        if (type instanceof WildcardType) {
            // todo handle this case
            return true;
        }
        return false;
    }

    /**
     * convert arbitraty string into an id
     *
     * @param englishText
     * @return
     */
    public static String textToKey(String englishText)
    {
        StringBuilder sb = new StringBuilder();
        boolean spaceSeen = false;
        for (int i = 0; i < englishText.length(); i++) {
            char c = englishText.charAt(i);
            if (Character.isJavaIdentifierPart(c)) {
                sb.append(c);
                continue;
            }
            if (spaceSeen)
                continue;
            spaceSeen = true;
            sb.append("_");

        }
        return sb.toString();
    }

    /**
     * convert NerdCaps to nerd caps
     *
     * @param englishText
     * @return
     */
    public static String camelCaseToWords(String englishText)
    {
        StringBuilder sb = new StringBuilder();
        boolean capsSeen = false;
        for (int i = 0; i < englishText.length(); i++) {
            char c = englishText.charAt(i);
            if (i > 0 && Character.isUpperCase(c)) {
                if (!capsSeen)
                    sb.append(" ");
                capsSeen = true;
                c = Character.toLowerCase(c);
            }
            else {
                capsSeen = false;
            }
            sb.append(c);

        }
        return sb.toString();
    }


    /**
     * Escape characters for text appearing as XML data, between tags.
     * <p/>
     * <P>The following characters are replaced with corresponding character entities :
     * <table border='1' cellpadding='3' cellspacing='0'>
     * <tr><th> Character </th><th> Encoding </th></tr>
     * <tr><td> < </td><td> &lt; </td></tr>
     * <tr><td> > </td><td> &gt; </td></tr>
     * <tr><td> & </td><td> &amp; </td></tr>
     * <tr><td> " </td><td> &quot;</td></tr>
     * <tr><td> ' </td><td> &#039;</td></tr>
     * </table>
     *
     * @see http://www.javapractices.com/topic/TopicAction.do?Id=96
     *      <P>Note that JSTL's {@code <c:out>} escapes the exact same set of
     *      characters as this method. <span class='highlight'>That is, {@code <c:out>}
     *      is good for escaping to produce valid XML, but not for producing safe
     *      HTML.</span>
     *      Because JavaRosa parser is poor at handling non-english characters as
     *      escape then all
     */
    public static String forXML(String aText)
    {
        final StringBuilder result = new StringBuilder();
        final StringCharacterIterator iterator = new StringCharacterIterator(aText);
        char character = iterator.current();
        while (character != CharacterIterator.DONE) {
            if (character == '<') {
                result.append("&lt;");
            }
            else if (character == '>') {
                result.append("&gt;");
            }
            else if (character == '\"') {
                result.append("&quot;");
            }
            else if (character == '\'') {
                result.append("&#039;");
            }
            else if (character == '&') {
                result.append("&amp;");
            }
            else if (Character.isDigit(character)) {
                result.append(character);
            }
            else {
                possiblyAppendEscape(result, character);
            }

            character = iterator.next();
        }
        return result.toString();
    }


    protected static void possiblyAppendEscape(StringBuilder result, char character)
    {
        if ((character >= 'a' && character <= 'z') ||
                (character >= 'A' && character <= 'Z')) {
            result.append(character);
        }
        else {
            switch (character) {
                case ' ':
                case '_':
                case '-':
                case '?':
                case '*':
                case '@':
                case '#':
                    result.append(character);
                    break;
                default:
                    String str = "&#x" + charToHex(character) + ";";
                    result.append(str);
            }

        }
    }

    // @See http://java.sun.com/docs/books/tutorial/i18n/text/examples/UnicodeFormatter.java

    static public String byteToHex(byte b)
    {
        // Returns hex String representation of byte b
        char hexDigit[] = {
                '0', '1', '2', '3', '4', '5', '6', '7',
                '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
        };
        char[] array = {hexDigit[(b >> 4) & 0x0f], hexDigit[b & 0x0f]};
        return new String(array);
    }

    static public String charToHex(char c)
    {
        // Returns hex String representation of char c
        byte hi = (byte) (c >>> 8);
        byte lo = (byte) (c & 0xff);
        return byteToHex(hi) + byteToHex(lo);
    }

}
