package com.lordjoe.formgenerator;

import java.lang.reflect.*;
import java.util.*;

/**
 * com.lordjoe.formgenerator.OptionSet
 *
 * @author Steve Lewis
 * @date Jan 31, 2010
 */
public class OptionSet implements IOptionSet
{
    public static OptionSet[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = OptionSet.class;

    public static Class[] ARG_TYPES = {};
    public static Object[] ARGS = {};

    private static IOptionSet BOOLEAN_SET;

    /**
     * return the values of an enum
     *
     * @param cls
     * @return
     */
    public static Object[] getEnumValues(Class cls)
    {
        if (!Enum.class.isAssignableFrom(cls))
            throw new IllegalArgumentException("Class " + cls + " is not an enum");
        try {
            Method met = cls.getMethod("values", ARG_TYPES);
            return (Object[]) met.invoke(null, ARGS);
        }
        catch (Exception e) {
            throw new IllegalStateException("problem"); // never get here
        }

    }


    public static synchronized IOptionSet getBooleanSet()
    {
        if (BOOLEAN_SET == null) {
            String[] opts = {"Yes", "No"};
            BOOLEAN_SET = buildStringOptions(opts);
        }

        return BOOLEAN_SET;
    }

    public static IOptionSet buildEnumStringOptions(Class cls)
     {
         OptionSet ret = new OptionSet();
         Object[] values = getEnumValues(cls);
         for (int i = 0; i < values.length; i++) {
             String value = values[i].toString();
             IOption opt = new StringOption(value);
             ret.addOption(opt);
         }
         return ret;
     }
    public static IOptionSet buildEnumINamedValueOptions(Class cls)
     {
         if(!(INamedValue.class.isAssignableFrom(cls)))
            throw new IllegalArgumentException("THis is only for INamedValue");
         OptionSet ret = new OptionSet();
         Object[] values = getEnumValues(cls);
         for (int i = 0; i < values.length; i++) {
             INamedValue o = (INamedValue)values[i];
              IOption opt = new IndexedOption(o.getName(), o.getValue());
             ret.addOption(opt);
         }
         return ret;
     }

    public static IOptionSet buildEnumIndexedOptions(Class cls)
    {
        OptionSet ret = new OptionSet();
        Object[] values = getEnumValues(cls);
        for (int i = 0; i < values.length; i++) {
            String value = values[i].toString();
            IOption opt = new IndexedOption(value, i);
            ret.addOption(opt);
        }
        return ret;
    }

    public static IOptionSet buildStringOptions(String[] values)
    {
        OptionSet ret = new OptionSet();
        for (int i = 0; i < values.length; i++) {
            String value = values[i];
            IOption opt = new StringOption(value);
            ret.addOption(opt);
        }
        return ret;
    }

    public static IOptionSet buildIndexedOptions(String[] values)
    {
        OptionSet ret = new OptionSet();
        for (int i = 0; i < values.length; i++) {
            String value = values[i];
            IOption opt = new IndexedOption(value, i);
            ret.addOption(opt);
        }
        return ret;
    }

    private final Map<String, IOption> m_IdToOption = new HashMap<String, IOption>();
    private final Map<Object, IOption> m_ValueToOption = new HashMap<Object, IOption>();
    private final List<IOption> m_Options = new ArrayList<IOption>();

    private OptionSet()
    {
    }

    private OptionSet(IOption[] opts)
    {
        this();
        for (int i = 0; i < opts.length; i++) {
            IOption opt = opts[i];
            addOption(opt);
            String id = opt.getId();
            if (m_IdToOption.containsKey(id))
                throw new IllegalStateException("duplicate id " + id);
            m_IdToOption.put(id, opt);
            Object value = opt.getValue();
            if (m_ValueToOption.containsKey(value))
                throw new IllegalStateException("duplicate value " + value);
            m_ValueToOption.put(value, opt);
        }
    }

    protected void addOption(IOption added)
    {
        m_Options.add(added);

    }

    public IOption[] getOptions()
    {
        return m_Options.toArray(IOption.EMPTY_ARRAY);
    }

    public IOption getOption(String id)
    {
        return m_IdToOption.get(id);
    }

    public IOption getOptionWithValue(String value)
    {
        return m_ValueToOption.get(value);
    }
}
