package com.lordjoe.formgenerator;

import java.lang.annotation.*;
import java.lang.reflect.*;

/**
 * com.lordjoe.formgenerator.Utilities
 *
 * @author Steve Lewis
 * @date Jan 18, 2010
 */
public class Utilities
{
    public static Utilities[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = Utilities.class;

    public static String getSurveyName(Class testClass) {
        SurveyName ann = (SurveyName)testClass.getAnnotation(SurveyName.class);
        String s = ann.Name();
        if(ann == null || s == null || s.length() == 0)
            return testClass.getSimpleName();
        return s;
    }

    public static String getPropertyPrefix(Class testClass) {
        PropertyPrefix ann = (PropertyPrefix)testClass.getAnnotation(PropertyPrefix.class);
        String s = ann.Prefix();
        if(ann == null || s == null || s.length() == 0)
            return null;
        return s;
    }
    public static FormModel fromClass(Class cls)
    {
        String prefix = "";
        PropertyPrefix pf = (PropertyPrefix) cls.getAnnotation(PropertyPrefix.class);
        if (pf != null)
            prefix = pf.Prefix();
        FormModel ret = new FormModel(cls.getSimpleName());
        for (Field f : cls.getDeclaredFields())
            addFieldProperties(ret, f, prefix);
        return ret;
    }

    protected static void addFieldProperties(AbstractFormGroup model, Field f, String prefix)
    {
        FormProperty prop = new FormProperty(model);
        prop.setName(buildFieldName(f, prefix));

        populateFormProperty(prop, f);
        model.addProperty(prop);
    }

    protected static void populateFormProperty(FormProperty prop, Field f)
    {
        Class<?> type = f.getType();
        FormType t = convertTypeToFormType(type, f);
        prop.setType(t);
        for (Annotation ann : f.getAnnotations()) {
            handleAnnotation(prop, ann);
        }

    }

    protected static void handleAnnotation(FormProperty prop, Annotation ann)
    {


    }

    protected static FormType convertTypeToFormType(Class<?> type, Field f)
    {
        AllowedType atype = AllowedType.String;
        if(type.isArray()) {
            atype = AllowedType.SelectMulti ;
         }
        if(type == Integer.class || type == Integer.TYPE ||
               type == Long.class || type == Long.TYPE  ) {
              atype = AllowedType.Number;
         }
        FormType ret = new FormType(atype);
           for (Annotation ann : f.getAnnotations()) {
             if(ann instanceof AllowedValues)    {
                   ret.addPropertyRestriction(new AnnotationPropertyRestriction(ann));
                 if(ret.getType() == AllowedType.SelectMulti)
                     continue;
                 ret.setType(AllowedType.Select1);
             }
             if(ann instanceof MinValue)
                   ret.addPropertyRestriction(new AnnotationPropertyRestriction(ann));
             if(ann instanceof MaxLength)
                   ret.addPropertyRestriction(new AnnotationPropertyRestriction(ann));
             if(ann instanceof MaxValue)
                   ret.addPropertyRestriction(new AnnotationPropertyRestriction(ann));

        }

         return ret;
    }


    protected static String buildFieldName(Field f, String prefix)
    {
        String name = f.getName();
        if (prefix == null || prefix.length() == 0)
            return name;
        if (name.startsWith(prefix))
            return name.substring(prefix.length());
        return name;

    }
}
