package com.lordjoe.measurement;

import com.lordjoe.formgenerator.*;

import java.util.*;

/**
 * com.lordjoe.measurement.UnitOfMeasure
 *
 * @author Steve Lewis
 * @date Mar 15, 2010
 */
@SimpleType
public abstract class BaseUnitOfMeasure<T extends IBaseUnit> implements IBaseUnit<T>
{


    private final String m_Name;
    private final List<IUnit<T>> m_Options = new ArrayList<IUnit<T>>();

    protected BaseUnitOfMeasure(String name)
    {
        m_Name = name;
    }

      /**
     * true if the units share the same base
     *
     * @param test non-null test
     * @return as above
     */
    public boolean isCompatable(IUnit<T> test)
    {
         return test.getBaseUnit() == getBaseUnit();
    }

    public void addOption(IUnit<T> added)
    {
        if(!added.isCompatable(this))
            throw new IllegalArgumentException("cannot add incompatible unit");
        m_Options.add(added);
    }

    public IUnit<T>[] getOptions()
    {
        return m_Options.toArray(IUnit.EMPTY_ARRAY);
    }

    public String getName()
    {
        return m_Name;
    }


    @Override
    public String toString()
    {
        return getName();

    }

    public T getBaseUnit()
    {
        return (T)this;
    }

    public double convertFromBase(double value)
    {
        return value;
    }


    public double convertToBase(double value)
    {
        return value;
    }


}