package com.lordjoe.measurement;

import com.lordjoe.formgenerator.*;

/**
 * com.lordjoe.measurement.UnitOfMeasure
 *
 * @author Steve Lewis
 * @date Mar 15, 2010
 */
@SimpleType
public class UnitOfMeasure<T extends BaseUnitOfMeasure> implements IUnit<T>
{

    private final String m_Name;
    private final T m_BaseUnit;
    private final double m_ConversionFactor;
    private final double m_ConversionOffset;

    public UnitOfMeasure(String name,T pBaseUnit, double pConversionFactor)
    {
        m_Name = name;
        m_BaseUnit = pBaseUnit;
        m_ConversionFactor = pConversionFactor;
        m_ConversionOffset = 0;
    }

     public UnitOfMeasure(String name,T pBaseUnit, double pConversionFactor,
                  double pConversionOffset)
    {
        m_Name = name;
        m_BaseUnit = pBaseUnit;
        m_ConversionFactor = pConversionFactor;
        m_ConversionOffset = pConversionOffset;
    }

    /**
     * true if the units share the same base
     *
     * @param test non-null test
     * @return as above
     */
    public boolean isCompatable(IUnit test)
    {
         return test.getBaseUnit() == getBaseUnit();
    }

    public String getName()
    {
        return m_Name;
    }

    public IUnit[] getOptions()
     {
         return m_BaseUnit.getOptions();
     }


    @Override
    public String toString()
    {
        return getName();

    }


    public T getBaseUnit()
    {
        return m_BaseUnit;
    }

    public double convertFromBase(double value)
    {
        value /= getConversionFactor();
        value += getConversionOffset();
        return value;
    }


    public double convertToBase(double value)
    {
        value -= getConversionOffset();
        value *= getConversionFactor();
        return value;
    }

    public double getConversionFactor()
    {
        return m_ConversionFactor;
    }

    public double getConversionOffset()
    {
        return m_ConversionOffset;
    }
}
