package com.lordjoe.pullparser;

import java.util.*;

/**
 * com.lordjoe.pullparser.ElementHandler
 *
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class ElementHandler<T> extends AbstractContextHandler<T>
{
    public static ElementHandler<?>[] EMPTY_ARRAY = {};
    // public static Class<ElementHandler> THIS_CLASS = ElementHandler.class;


    public static <T> ElementHandler<T> getNullHandler()
    {
        return new Nullhandler<T>();
    }

    private final String m_ElementName;
    private final Map<String, ElementHandler<T>> m_SubHandlers = new HashMap<String, ElementHandler<T>>();

    public ElementHandler(String pElementName)
    {
        m_ElementName = pElementName;
    }

    public String getElementName()
    {
        return m_ElementName;
    }


    public void addElementHandler(ElementHandler<T> e)
    {
        m_SubHandlers.put(e.getElementName(), e);
    }

  

    @Override
    public ElementHandler<T> getElementHandler(String prefix, String localName)
    {
        return m_SubHandlers.get( buildFullName(prefix,localName));
    }


    private static class Nullhandler<T> extends ElementHandler<T>
    {
        private Nullhandler()
        {
            super("Null Handler");
        }

        @Override
        public ElementHandler<T> getElementHandler(String prefix, String localName)
        {
            return null;
        }   
    }

}
