package com.lordjoe.pullparser;

import javax.xml.namespace.*;
import javax.xml.stream.*;
import java.io.*;

/**
 * com.lordjoe.pullparser.JavaPullParser
 *  NOTE the Androis uses a different library
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class JavaPullParser<T> implements IPullParser<T>
{
    public static JavaPullParser[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = JavaPullParser.class;

    private final DocumentHandler<T> m_Handler;

    public JavaPullParser(DocumentHandler<T> pHandler)
    {
        m_Handler = pHandler;
    }

    public DocumentHandler getHandler()
    {
        return m_Handler;
    }

    public T parseXMLDocument(String pInput) {
            return parseReader(new StringReader(pInput));
    }

    public T parseXMLDocument(File pInput) {
        try {
            return parseReader( new InputStreamReader(new FileInputStream(pInput),"UTF8"));

        }
        catch(IOException ex)  {
            throw new RuntimeException(ex);
        }
    }

    /**
     * parse the contents as Stream
     *
     * @param inp
     * @return
     */
    public T parseXMLDocument(InputStream inp) {
        Reader input = new InputStreamReader(inp);
        return parseReader(input);

    }
    

  public T parseReader(Reader pInput) {
		DocumentHandler<T> documentHandler = getHandler();
		ParseContext<T> context = documentHandler.buildContext();
		/*
		 * Ensure the factory implementation is loaded from the application
		 * classpath (which contains the implementation classes), rather than
		 * the system classpath (which doesn't).
		 */
		try {
            XMLInputFactory inputFactory = XMLInputFactory.newInstance();
              XMLStreamReader xpp = inputFactory.createXMLStreamReader(pInput);
             context = documentHandler.buildContext();
			int event = xpp.getEventType();

			while (event != XMLStreamConstants.END_DOCUMENT) {

				if (event == XMLStreamConstants.START_DOCUMENT) {
					event = xpp.next();
					continue;
				}
				if (event == XMLStreamConstants.START_ELEMENT) {
                    QName qName = xpp.getName();
                    String prefix = qName.getPrefix();
					String localName = qName.getLocalPart();
					// String namespace = xpp.getNamespace();
					IContextHandler<T> contextHandler = context.getHandler();
					IContextHandler<T> handler = contextHandler
							.getElementHandler(prefix, localName);
					if (handler == null)
						handler = contextHandler;
					context.pushHandler(handler);

					for (int i = 0; i < xpp.getAttributeCount(); i++) {
                        QName aqName = xpp.getAttributeName(i);
                          String aprefix = aqName.getPrefix();
           				// String aPrefix = xpp.getAttributePrefix(i);
						String alocalName = aqName.getLocalPart();
						String val = xpp.getAttributeValue(i);
						context.setValue(alocalName, val);
					}
                    contextHandler.onOpen(context);
					event = xpp.next();
					continue;
				}

				if (event == XMLStreamConstants.CHARACTERS) {
					String val = xpp.getText();
					if (val.trim().length() > 0)
						context.addText(val);
					event = xpp.next();
					continue;
				}
				if (event == XMLStreamConstants.COMMENT) {
					String comment = xpp.getText();
					context.getCurrentLevel().addComment(comment);
					event = xpp.next();
					continue;
				}
				if (event == XMLStreamConstants.END_ELEMENT) {
					IContextHandler<T> contextHandler = context.getHandler();
					// are we reusing the handler on the next level??
					ParseContextLevel<T> currentlevel = context
							.getCurrentLevel();
					ParseContextLevel<T> parentLevel = currentlevel.getParent();
					if (parentLevel == null
							|| parentLevel.getHandler() != contextHandler)
						contextHandler.onClose(context);
					context.popLevel();
					event = xpp.next();
					continue;
				}

				if (event == XMLStreamConstants.PROCESSING_INSTRUCTION) {
					event = xpp.next();
					continue;
				}
				event = xpp.next();
			}

			IContextHandler<T> contextHandler = context.getHandler();
			contextHandler.onClose(context);

			return context.getActiveDocument();
		}

		catch (XMLStreamException e) {
			throw new RuntimeException(e);
		} catch (RuntimeException e) {
			throw e;
		}
	}

//    public T parseReader(File inp)
//    {
//        DocumentHandler documentHandler = getHandler();
//        ParseContext<T> context = null;
//        try {
//            XMLInputFactory inputFactory = XMLInputFactory.newInstance();
//            InputStream input = new FileInputStream(inp);
//            XMLStreamReader xmlStreamReader = inputFactory.createXMLStreamReader(input);
//            context = documentHandler.buildContext();
//
//            while (xmlStreamReader.hasNext()) {
//                int event = xmlStreamReader.next();
//
//                if (event == XMLStreamConstants.START_DOCUMENT) {
//                     continue;
//                }
//                if (event == XMLStreamConstants.START_ELEMENT) {
//                    String prefix = xmlStreamReader.getPrefix();
//                    String localName = xmlStreamReader.getLocalName();
//                    String namespace = xmlStreamReader.getNamespaceURI();
//                    IContextHandler<T> contextHandler = context.getHandler();
//                    ElementHandler<T> handler = contextHandler.getElementHandler(prefix,localName);
//                    if (handler == null)
//                        handler = ElementHandler.getNullHandler();
//                    context.pushHandler(handler);
//                   // System.out.println("Event Type: START_ELEMENT");
//                   // System.out.println("Namespace URI:" + namespace );
//                   // System.out.println("Element Local Name:" + localName);
//                   // System.out.println("Element  Prefix:" + prefix);
//
//                    for (int i = 0; i < xmlStreamReader.getAttributeCount(); i++) {
//                        String aPrefix = xmlStreamReader.getAttributePrefix(i);
//                        String alocalName = xmlStreamReader.getAttributeLocalName(i);
//                        String val = xmlStreamReader.getAttributeValue(i);
//                        IContextHandler<T> hdlr = context.getHandler();
//                        AttributeHandler<T> ahandler = handler.getAttributeHandler(aPrefix, alocalName,val);
//                        if (ahandler == null)
//                            continue;
//                        context.setValue("AttributeNme",val);
//                        ahandler.onClose(context);
//                     }
//
//                    continue;
//                    }
//                if (event == XMLStreamConstants.ATTRIBUTE) {
//                   // System.out.println("Event Type:ATTRIBUTE");
//                    continue;
//                    }
//
//                if (event == XMLStreamConstants.CHARACTERS) {
//                //    System.out.println("Event Type: CHARACTERS");
//                    String val = xmlStreamReader.getText();
//               //     System.out.println("Text:" + val);
//                    if(val.trim().length() > 0)
//                        context.addText(val);
//                    continue;
//                 }
//                if (event == XMLStreamConstants.COMMENT) {
//                  //  System.out.println("Event Type:COMMENT");
//                    String comment = xmlStreamReader.getText();
//                  //  System.out.println("Comment Text:" + comment);
//                    continue;
//                    }
//
//                if (event == XMLStreamConstants.END_DOCUMENT) {
//                    IContextHandler<T> contextHandler = context.getHandler();
//                    contextHandler.onClose(context);
//
//                    return context.getActiveDocument();
//                     }
//                if (event == XMLStreamConstants.END_ELEMENT) {
//                    IContextHandler<T> contextHandler = context.getHandler();
//                    contextHandler.onClose(context);
//                    context.popHandler();
//                    continue;
//                    }
//
//                if (event == XMLStreamConstants.NAMESPACE) {
//                 //   System.out.println("Event Type:NAMESPACE");
//                    continue;
//                    }
//
//                if (event == XMLStreamConstants.PROCESSING_INSTRUCTION) {
//                   // System.out.println("Event Type: PROCESSING_INSTRUCTION");
//
//                   // System.out.println("PI Target:" + xmlStreamReader.getPITarget());
//                   // System.out.println("PI Data:" + xmlStreamReader.getPIData());
//                    continue;
//                    }
//                if (event == XMLStreamConstants.SPACE) {
//                  //  System.out.println("Event Type: SPACE");
//                  //  System.out.println("Text:" + xmlStreamReader.getText());
//
//                    continue;
//                    }
//
//            }
//        }
//        catch (FactoryConfigurationError e) {
//            throw new RuntimeException(e);
//        }
//        catch (XMLStreamException e) {
//            throw new RuntimeException(e);
//        }
//        catch (IOException e) {
//            throw new RuntimeException(e);
//        }
//        IContextHandler<T> contextHandler = context.getHandler();
//            contextHandler.onClose(context);
//
//            return context.getActiveDocument();
//
//    }
}
