package com.lordjoe.pullparser;

import java.util.*;

/**
 * com.lordjoe.pullparser.ParseContext
 *
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class ParseContext<T>
{
    public static ParseContext<?>[] EMPTY_ARRAY = {};
    // public static Class THIS_CLASS = ParseContext.class;

    private final T m_ActiveDocument;
    private final Stack<ParseContextLevel<T>> m_Level = new Stack<ParseContextLevel<T>>();

    public ParseContext(T pActiveDocument, DocumentHandler<T> handler)
    {
        m_ActiveDocument = pActiveDocument;
        pushHandler(handler);
    }

    public T getActiveDocument()
    {
        return m_ActiveDocument;
    }

    public ParseContextLevel<T> getCurrentLevel()
    {
        if(m_Level.isEmpty())
            return null;
        return m_Level.peek();
    }

    public IContextHandler<T> getHandler()
    {
        return getCurrentLevel().getHandler();
    }

    public void pushHandler(IContextHandler<T> h)
    {
        m_Level.push(new ParseContextLevel<T>(getCurrentLevel(), h));

    }

    public void addText(String text)
    {
        getCurrentLevel().addText(text);
    }

    public String getText()
    {
        return getCurrentLevel().getText();
    }

    public Object getValue(String key)
    {
        return getCurrentLevel().getValue(key);
    }

    public void setValue(String key, Object value)
    {
        getCurrentLevel().setValue(key, value);
    }

    public void popLevel()
    {
        m_Level.pop();
    }
}
