package com.lordjoe.translation;

import com.google.api.translate.*;

import java.io.*;
import java.util.*;

/**
 * com.lordjoe.translation.AndroidImporter
 *
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class AndroidImporter
{
    public static AndroidImporter[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = AndroidImporter.class;

    private final File m_BaseDirectory;

    public AndroidImporter(File pBaseDirectory)
    {
        m_BaseDirectory = pBaseDirectory;
    }


    public File getBaseDirectory()
    {
        return m_BaseDirectory;
    }

    public void reconcileLanguageBundle(File bundleDirectory, String baseName)
    {
        if (!bundleDirectory.isDirectory() && !bundleDirectory.exists() && !bundleDirectory.mkdirs())
            throw new IllegalStateException(
                    "Cannot create bundle directory " + bundleDirectory.getAbsolutePath());
        Map<Language, LanguageBundle> androidResources = readResourceDirectories();
        ResourceTranslator rt = new ResourceTranslator(bundleDirectory);
        rt.setBaseName(baseName);
        Locale loc = Locale.getDefault();
        Language defaultLanguage = localeToLanguage(loc);
        File bunfleFile = rt.buildResourcePropertiesFile(defaultLanguage);
        if (!bunfleFile.exists()) {
            androidResources.get(defaultLanguage).save(bunfleFile);
        }

        List<Language> holder = new ArrayList<Language>();
        for (Language l : androidResources.keySet()) {
            holder.add(l);
            LanguageBundle lb = androidResources.get(l);
            guaranteeBundle(lb, rt);
        }
        Language[] langs = new Language[holder.size()];
        holder.toArray(langs);
        rt.translateDirectory(langs);
        for (Language l : androidResources.keySet()) {
            holder.add(l);
            LanguageBundle lb = androidResources.get(l);
            stringsFromBundle(lb, rt);
            saveAndroidStrings(lb);
        }

    }

    protected void stringsFromBundle(LanguageBundle pLb, ResourceTranslator rt)
    {
        Properties props = rt.getLanguageKeys(pLb.getLanguage());
        for (String key : props.stringPropertyNames()) {
            String val = props.getProperty(key);
            pLb.setProperty(key, val);
        }
    }


    protected void guaranteeBundle(LanguageBundle pLb, ResourceTranslator rt)
    {
        File bunfleFile = rt.buildResourcePropertiesFile(pLb.getLanguage());
        if (!bunfleFile.exists()) {
            pLb.save(bunfleFile);
        }
        else {

        }
    }

    public Map<Language, LanguageBundle> readResourceDirectories()
    {
        Map<Language, LanguageBundle> ret = new HashMap<Language, LanguageBundle>();
        Map<Language, File> dirs = new HashMap<Language, File>();
        File base = getBaseDirectory();
        String[] fileNames = base.list();
        if (fileNames == null)
            throw new IllegalStateException("problem");
        boolean valueSeen = false;
        for (int i = 0; i < fileNames.length; i++) {
            String fileName = fileNames[i];
            if (!fileName.startsWith("values"))
                continue;
            File f = new File(base, fileName);
            Language l = null;
            if (fileName.equals("values")) {
                Locale loc = Locale.getDefault();
                l = localeToLanguage(loc);
                dirs.put(l, f);
            }
            else {
                if (f.isDirectory() && fileName.startsWith("values-")) {
                    String shortLanguage = fileName.substring("values-".length());
                    l = shortNameToLanguge(shortLanguage);
                }
                if (l == null)
                    throw new UnsupportedOperationException("Fix This"); // ToDo
                dirs.put(l, f);

            }
        }
        for (Language l : dirs.keySet()) {
            LanguageBundle value = readProperties(l, dirs.get(l));
            ret.put(l, value);
        }
        return ret;
    }

    public File languageResourceDirectory(Language l)
    {
        File base = getBaseDirectory();
        Locale loc = Locale.getDefault();
        if (l == localeToLanguage(loc))
            return new File(base, "values");
        else
            return new File(base, "values-" + l.toString());
    }

    public File languageStringsFile(Language l)
    {
        File base = languageResourceDirectory(l);
        base.mkdirs();
        return new File(base, "strings.xml");
    }

    private LanguageBundle readProperties(Language l, File pFile)
    {
        File stringsFile = new File(pFile, "strings.xml");
        if (stringsFile.exists()) {
            LanguageBundle pc = AndroidStringParser.HANDLER.parse(stringsFile);
            pc.setLanguage(l);
            pc.setSource(pFile);

            return pc;
        }
        else {
            return new LanguageBundle(l, stringsFile);
        }
    }

    public void saveAndroidStrings(LanguageBundle lb)
    {
        Language l = lb.getLanguage();
        File file = languageStringsFile(l);
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new OutputStreamWriter(new FileOutputStream(file),"UTF8"));
            pw.println("<?xml version=\"1.0\" encoding=\"utf-16\"?>");
            pw.println("<resources>");
            Map<String, List<String>> arrays = new HashMap<String, List<String>>();

            for (String key : lb.getKeys()) {
                if (key.startsWith(AndroidStringParser.ARRAY_NAME)) {
                    handleArrayElement(key, lb.getProperty(key), arrays);
                    continue;
                }
                String val = lb.getProperty(key);
                pw.println("    <string name=\"" + key + "\">" + val + "</string>");
            }
            for (String key : arrays.keySet()) {
                List<String> items = arrays.get(key);
                pw.println("    <string-array name=\"" + key + "\">");
                for(String s : items)
                     pw.println("        <item>"  + s + "</item>");
                pw.println("    </string-array>");
            }

            pw.println("</resources>");
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
        finally {
            if (pw != null)
                pw.close();
        }

    }

    private void handleArrayElement(String pKey, String pProperty,
                                    Map<String, List<String>> pArrays)
    {
        pKey = pKey.substring(AndroidStringParser.ARRAY_NAME.length());
        int end = pKey.indexOf("|");
        String arrayName = pKey.substring(0, end);
        List<String> lst = pArrays.get(arrayName);
        if(lst == null)  {
            lst = new ArrayList<String>();
            pArrays.put(arrayName,lst);
        }
        lst.add(pProperty);
    }

    public static Language nameToLanguge(String pLoc)
    {
        Language[] languages = Language.values();
        for (int i = 0; i < languages.length; i++) {
            Language language = languages[i];
            String ln = language.name();
            if (pLoc.startsWith(ln))
                return language;
        }
        return null;
    }

    public static Language shortNameToLanguge(String pLoc)
    {
        Language[] languages = Language.values();
        for (int i = 0; i < languages.length; i++) {
            Language language = languages[i];
            String ln = language.toString();
            if (ln.length() == 0)
                continue;
            if (pLoc.startsWith(ln))
                return language;
        }
        return null;
    }

    public static Language localeToLanguage(Locale pLoc)
    {
        String s = pLoc.getDisplayLanguage();
        Language language = nameToLanguge(s);
        if (language != null)
            return language;
        language = nameToLanguge(s.toUpperCase());
        if (language != null)
            return language;
        language = findNamedLanguage(s.toUpperCase());
        return language;
    }

    public static Language findNamedLanguage(String pS)
    {
        Language[] languages = Language.values();
        for (int i = 0; i < languages.length; i++) {
            Language language = languages[i];
            String ln = language.name();
            if (pS.equalsIgnoreCase(ln))
                return language;
        }
        return null;
    }

        private static void printLanguages()
    {
        Language[] languages = Language.values();
        for (int i = 0; i < languages.length; i++) {
            Language language = languages[i];
            System.out.print("\"" + language.name() + "\",");
        }
        System.out.println();
        for (int i = 0; i < languages.length; i++) {
            Language language = languages[i];

            System.out.print("\"" + language.toString().toLowerCase() + "\",");
        }
        System.out.println();
    }

    public static void main(String[] argv)
    {
        // Usage AndroidImporter
        String s = argv[0];
        File baseDirectory = new File(s);
        AndroidImporter ai = new AndroidImporter(baseDirectory);
        Map<Language, LanguageBundle> map = ai.readResourceDirectories();
        File arent = baseDirectory.getParentFile();
        ai.reconcileLanguageBundle(new File(arent, "bundle"), arent.getName());
        for (Language l : map.keySet()) {
            LanguageBundle lb = map.get(l);
            for (String key : lb.getKeys()) {
                String val = lb.getProperty(key);
                val = null;
            }
            lb = null;
        }


        //    xmlParser.parseXMLDocument();

    }


}
