package com.lordjoe.translation;

import com.google.api.translate.*;
import com.lordjoe.pullparser.*;

import javax.xml.stream.*;
import javax.xml.stream.XMLInputFactory;
import java.io.*;
import java.util.*;

/**
 * com.lordjoe.translation.AndroidStringParser
 *
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class AndroidStringParser
{

    public static final String ARRAY_NAME = "__Array|";

    public static DocumentHandler<LanguageBundle> HANDLER =
            new StringResourceParser();

    protected static class StringResourceParser extends DocumentHandler<LanguageBundle>
    {
        public StringResourceParser()
        {
            addElementHandler(new StringHandler());
            addElementHandler(new StringArrayHandler());
        }

        @Override
        public ParseContext<LanguageBundle> buildContext()
        {
            return new ParseContext<LanguageBundle>(new LanguageBundle(null, null), this);
        }
    }


    protected static class StringHandler extends ElementHandler<LanguageBundle>
    {
        public StringHandler()
        {
            super("string");
        }

        @Override
        public void onClose(ParseContext<LanguageBundle> ctx)
        {
            LanguageBundle document = ctx.getActiveDocument();
            ParseContextLevel<LanguageBundle> lvl = ctx.getCurrentLevel();
            Object name = ctx.getValue("name");
            String s = name.toString();
            String txt = ctx.getText();
            document.setProperty(s, txt);
        }
    }

    protected static class StringArrayHandler extends ElementHandler<LanguageBundle>
    {
        public StringArrayHandler()
        {
            super("string-array");
            addElementHandler(new ItemHandler());
        }

        @Override
        public void onOpen(ParseContext<LanguageBundle> ctx)
        {
            super.onOpen(ctx);
            ctx.setValue("index", new Integer(0));

        }

        @Override
        public void onClose(ParseContext<LanguageBundle> ctx)
        {
            LanguageBundle document = ctx.getActiveDocument();
            ParseContextLevel<LanguageBundle> lvl = ctx.getCurrentLevel();
        }
    }


    protected static class ItemHandler extends ElementHandler<LanguageBundle>
    {
        public ItemHandler()
        {
            super("item");
        }

        @Override
           public void onOpen(ParseContext<LanguageBundle> ctx)
           {
               super.onOpen(ctx);

           }

        @Override
        public void onClose(ParseContext<LanguageBundle> ctx)
        {
            LanguageBundle document = ctx.getActiveDocument();
            ParseContextLevel<LanguageBundle> lvl = ctx.getCurrentLevel();
              String txt = ctx.getText();
            ParseContextLevel<LanguageBundle> arLevel = ctx.getCurrentLevel().getParent();
            String name = (String) arLevel.getValue("name");
            Integer idx = (Integer) arLevel.getValue("index");
            if(idx == null) {
                idx = 0;
            }
            String s = ARRAY_NAME + name + "|" + idx;
            document.setProperty(s, txt);
            arLevel.setValue("index", new Integer(idx + 1));

        }
    }


    public static void main(String[] argv)
    {

        //    AndroidStringParser xmlParser = new AndroidStringParser();
        //    xmlParser.parseXMLDocument();

    }
}
