package com.lordjoe.translation;

import com.google.api.translate.*;

import java.io.*;
import java.util.*;

/**
 * com.lordjoe.translation.LanguageBundle
 *     More than a properties file it tracks whether the translations are automatic
 * @author Steve Lewis
 * @date Feb 2, 2010
 */
public class LanguageBundle
{
    public static LanguageBundle[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = LanguageBundle.class;

    public static final String AUTO_GENERATED_COMMENT =
            "# The next entry is AutoGenerated";

    private Language m_Language;
    private File m_Source;
    private final Map<String, TranslationEntry> m_Properties = new
            HashMap<String, TranslationEntry>();

    public LanguageBundle(Language pLanguage, File pSource)
    {
        m_Language = pLanguage;
        m_Source = pSource;
    }

    public Language getLanguage()
    {
        return m_Language;
    }

    public void setLanguage(Language pLanguage)
    {
        m_Language = pLanguage;
    }

    public String[] getKeys()
     {
         String[] strings = m_Properties.keySet().toArray(new String[0]);
         Arrays.sort(strings);
         return strings;
     }
    public File getSource()
     {
         return m_Source;
     }

    public void setSource(File pSource)
    {
        m_Source = pSource;
    }

    public void load(File f)
    {
        setSource(f);
        load();
    }

    public void save(File f)
    {
        setSource(f);
        save();
    }

    public String getProperty(String key)
    {
        TranslationEntry ti = getTranslationEntry(key);
        if (ti == null)
            return null;
        return ti.getValue();
    }

    public void setProperty(String key,String val)
    {
        TranslationEntry ti = getTranslationEntry(key);
        if(ti == null)  {
             ti = new TranslationEntry(key);
            ti.setValue(val);
            ti.setHandEdited(true);
            m_Properties.put(key,ti);
        }
        else {
            ti.setValue(val);
             ti.setHandEdited(true);

        }
    }

    public TranslationEntry getTranslationEntry(String key)
    {
        TranslationEntry ti = m_Properties.get(key);
        return ti;
    }

    public void addTranslationEntry(TranslationEntry entry)
    {
        m_Properties.put(entry.getKey(),entry);
     }

    public void save()
    {
        try {
            PrintWriter out = new PrintWriter(new FileWriter(getSource()));
            String[] keys = m_Properties.keySet().toArray(new String[0]);
            Arrays.sort(keys);
            for (int i = 0; i < keys.length; i++) {
                String key = keys[i];
                TranslationEntry ti = m_Properties.get(key);
                if (!ti.isHandEdited())
                    out.println(AUTO_GENERATED_COMMENT);
                out.print(key);
                out.print("=");
                out.println(ti.getValue());

            }
            out.close();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void load()
    {
         try {
            File source = getSource();
            if(!source.exists())
                return;
             m_Properties.clear();
            LineNumberReader inp = new LineNumberReader(new InputStreamReader(new FileInputStream(source),"UTF8"));
            boolean autogenerated = false;
            String line = inp.readLine();
            while (line != null) {
                if (line.equals(AUTO_GENERATED_COMMENT)) {
                    autogenerated = true;
                    line = inp.readLine();
                    continue;
                }
                if (line.startsWith("#"))
                    continue;
                String[] values = line.split("=");
                TranslationEntry te = new TranslationEntry(values[0]);
                te.setValue(values[1]);
                te.setHandEdited(autogenerated == false);
                addTranslationEntry(te);
                autogenerated = false;
                line = inp.readLine();
            }
            inp.close();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

    }
}
