package com.lordjoe.translation;

import com.google.api.translate.*;
import com.lordjoe.pullparser.*;

import java.io.*;

/**
 * com.lordjoe.translation.LanguageBundleDocumentHandler
 *   handle android Strings.xml
 * @author Steve Lewis
 * @date Feb 10, 2010
 */
public class LanguageBundleDocumentHandler extends DocumentHandler<LanguageBundle>
{
    public static LanguageBundleDocumentHandler[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = LanguageBundleDocumentHandler.class;

    private final File m_Source;
    private final Language m_Language;

    public LanguageBundleDocumentHandler(File pSource, Language pLanguage)
    {
        m_Source = pSource;
        m_Language = pLanguage;
        initializeHandlers();
    }

    public File getSource()
    {
        return m_Source;
    }

    public Language getLanguage()
    {
        return m_Language;
    }

    @Override
    public ParseContext<LanguageBundle> buildContext()
    {
        Language language = getLanguage();
        File source = getSource();
        LanguageBundle lb = new LanguageBundle(language, source);
        ParseContext<LanguageBundle> ln = new ParseContext<LanguageBundle>(lb,this);

        return ln;
    }

    protected void initializeHandlers() {
        ElementHandler<LanguageBundle> res =
                new ElementHandler<LanguageBundle>("resources");
        TranslationEntryHandler  string =
                  new TranslationEntryHandler();
      res.addElementHandler(string);

        addElementHandler(res);
    }

    public static void main(String[] args)
    {
        File source = new File("values/Strings.xml");
        LanguageBundleDocumentHandler ld = new LanguageBundleDocumentHandler(source, Language.ENGLISH);
        IPullParser< LanguageBundle> pp = new JavaPullParser< LanguageBundle>(ld);
        LanguageBundle o = pp.parseXMLDocument(ld.getSource());
        o = null;
    }
}
