package forms;

import com.google.api.translate.*;
import com.lordjoe.formgenerator.*;
import com.lordjoe.measurement.*;

import java.util.*;

/**
 * com.lordjoe.testclasses.Student1
 *
 * @author Steve Lewis
 * @date Jan 18, 2010
 */
@PropertyPrefix(Prefix = "m_")
@SurveyName(Name = "Medical Survey")
@SupportedLanguages(Languages = { Language.SPANISH,Language.ENGLISH,Language.SWAHILI})
public class MedicalForm
{

    @Required
    @MaxLength(Max = 64)
    private String m_Name;
    @Required
    private Sex m_Sex;
    @RelevantIf(Expression = "selected(/Medical_Survey/Sex, \'Female\')")
    private FemaleQuestions m_FemaleQuestions;
    @RelevantIf(Expression = "selected(/Medical_Survey/Sex, \'Male\')")
    private MaleQuestions m_MaleQuestions;
    @Question(Id = "Have you had sexual relations")
    private boolean m_SexualRelations;
    @RelevantIf(Expression = "SexualRelations=true")
    @AsOneScreen
    @Question(Id = "How long ago was the last time you had sexual relations")
    private Measurement<LongTimeUnits> m_LastRelations = new Measurement<LongTimeUnits>(LongTimeUnits.INSTANCE);

    @Required
    private Date m_Birthday;

    public class FemaleQuestions
    {
        @Question(Id = "Have you had your first period")
        private Boolean m_StartedMenstruating;
        @RelevantIf(Expression = "StartedMenstruating=true")
        @Question(Id = "Are You Pregnant")
        private Boolean m_Pregnant;
        @Question(Id = "At what age did you start menstruating")
        @MaxValue(Max = 25)
        @MinValue(Min = 8)
        private Integer m_StartMensesAge;
        private List<ChildQuestions> m_Child; // singular


    }

    public static enum Sex { Male,Female}

    @PreferedPronoun(Pronoun = QuestionPossessive.His)
    public class ChildQuestions
    {
        @Required
        @MaxLength(Max = 64)
        private String m_Name;
        @Required
        private Sex m_Sex;
        @Required
        private int m_Age;
    }

    public class MaleQuestions
    {
        @Question(Id = "Are you Circumcised")
        private Boolean m_Circumcised;
        @Question(Id = "How often do you Masturbate")
        private MasturbationFrequency m_MasterbationRate;
    }

    public enum MasturbationFrequency implements INamedValue
    {
        Never(0),
        Monthly(1),
        Weekly(2),
        EveryOtherDay(3),
        Daily(4),
        MultipleTimesDaily(5);


        private final Integer m_Value;

        private MasturbationFrequency(int pValue)
        {
            m_Value = pValue;
        }


        public Object getValue()
        {
            return m_Value;
        }

        public String getName()
        {
            return toString();
        }
    }
}