package com.lordjoe.testclasses;

import com.google.api.translate.*;
import com.lordjoe.formgenerator.*;
import org.junit.*;

import java.io.*;

/**
 * com.lordjoe.testclasses.TestGenerator
 *
 * @author Steve Lewis
 * @date Jan 29, 2010
 */
public class TestGenerator
{
    public static TestGenerator[] EMPTY_ARRAY = {};
    public static Class THIS_CLASS = TestGenerator.class;

    @Test
    public void testModelGenerator()
    {
        FormModelGenerator gen = FormModelGenerator.INSTANCE;
        gen.setPrefix("m_");
        String s = "Student Registration";
        String form = buildForm(gen, s, Student1.class);
        System.out.println(form);
    }

    @Test
    public void testSamples()
    {
        FormModelGenerator gen = FormModelGenerator.INSTANCE;
        gen.setPrefix("m_");
        String s = "ImageQuestion";
        Class aClass = ImageSurvey.class;
        String form = buildForm(gen, s, aClass);
        System.out.println(form);

        s = "MonthQuestion";
        aClass = MonthSurvey.class;
        form = buildForm(gen, s, aClass);
        System.out.println(form);

        s = "CompositeQuestion";
        aClass = CompositeSurvey.class;
        form = buildForm(gen, s, aClass);
        System.out.println(form);
    }

    private String buildForm(FormModelGenerator pGen, String pS, Class pAClass)
    {
        FormModel formModel = pGen.generateModel(pAClass, pS);
        Language[] langs = {Language.ENGLISH, Language.GERMAN, Language.TURKISH, Language.SPANISH};
        String form = formModel.generateXForm(langs);
        File formsDirectory = getFormsDirectory();
        writeForm(formsDirectory, pS, form);
        return form;
    }

    @Test
    public void testStudent2Generator()
    {
        FormModelGenerator gen = FormModelGenerator.INSTANCE;
        gen.setPrefix("m_");
        String s = "Student2 Registration";
        FormModel formModel = gen.generateModel(Student2.class, s);
        Language[] langs = {Language.SPANISH, Language.ENGLISH, Language.GERMAN, Language.TURKISH};
        String form = formModel.generateXForm(langs);
        File formsDirectory = getFormsDirectory();
        writeForm(formsDirectory, s, form);
        System.out.println(form);

    }

    @Test
    public void testNameSurvey()
    {
        FormModelGenerator gen = FormModelGenerator.INSTANCE;
        gen.setPrefix("m_");
        String s = "Name Survey";
        FormModel formModel = gen.generateModel(NameSurvey.class, s);
        Language[] langs = {Language.ENGLISH, Language.GERMAN, Language.TURKISH, Language.SPANISH};
        String form = formModel.generateXForm(langs);
        File formsDirectory = getFormsDirectory();
        writeForm(formsDirectory, s, form);
        System.out.println(form);

    }

    public static File getFormsDirectory()
    {
        File udir = new File(System.getProperty("user.dir"));
        File formsdDir = new File(udir, "forms");
        formsdDir.mkdirs();
        return formsdDir;

    }

    public static void writeForm(File formsDirectory, String name, String text)
    {
        try {
            PrintWriter out = new PrintWriter(
                    new FileWriter(new File(formsDirectory, name + ".xml")));
            out.println(text);
            out.close();
        }
        catch (IOException e) {
            throw new RuntimeException(e);
        }

    }
}
